@@ -11,7 +11,7 @@ import sqlalchemy as sql
 from aiojobs.aiohttp import spawn
 from aiohttp import web
 from cryptography.fernet import InvalidToken
-from marshmallow import fields, validate, validates, ValidationError
+from marshmallow import fields, validate, validates, ValidationError, Schema
 from psycopg2 import errors as postgres_errors
 from sqlalchemy.dialects.postgresql import JSONB
 
@@ -31,7 +31,8 @@ from .utils import (
     parse_postgres_err,
     seconds_to_human,
     Json,
-    dumps
+    dumps,
+    interpolate_env_var
 )
 from .scope import ScopeBase
 from .view import AuxView
@@ -84,10 +85,10 @@ async def send_email_user_invitation(request, by, link, to):
 
     await aiosmtplib.send(
         message,
-        hostname=os.environ.get('EMAIL_HOSTNAME'),
+        hostname=interpolate_env_var('EMAIL_HOSTNAME'),
         use_tls=True,
-        username=os.environ.get('EMAIL_USERNAME'),
-        password=os.environ.get('EMAIL_PASSWORD')
+        username=interpolate_env_var('EMAIL_USERNAME'),
+        password=interpolate_env_var('EMAIL_PASSWORD')
     )
     request.app.info_logger.info("Invitation email sent", invited=to)
 
@@ -129,10 +130,10 @@ async def send_email_reset_link(request, checkcode, to):
 
     await aiosmtplib.send(
         message,
-        hostname=os.environ.get('EMAIL_HOSTNAME'),
+        hostname=interpolate_env_var('EMAIL_HOSTNAME'),
         use_tls=True,
-        username=os.environ.get('EMAIL_USERNAME'),
-        password=os.environ.get('EMAIL_PASSWORD')
+        username=interpolate_env_var('EMAIL_USERNAME'),
+        password=interpolate_env_var('EMAIL_PASSWORD')
     )
     request.app.info_logger.info("Sent password reset link", email=to)
 
@@ -171,10 +172,10 @@ async def send_email_verification_link(request, to):
 
     await aiosmtplib.send(
         message,
-        hostname=os.environ.get('EMAIL_HOSTNAME'),
+        hostname=interpolate_env_var('EMAIL_HOSTNAME'),
         use_tls=True,
-        username=os.environ.get('EMAIL_USERNAME'),
-        password=os.environ.get('EMAIL_PASSWORD')
+        username=interpolate_env_var('EMAIL_USERNAME'),
+        password=interpolate_env_var('EMAIL_PASSWORD')
     )
     request.app.info_logger.info("Sent email verification link", email=to)
 
@@ -216,10 +217,10 @@ async def send_email_activation_link(request, data, link_path_base, ttl_seconds)
 
     await aiosmtplib.send(
         message,
-        hostname=os.environ.get('EMAIL_HOSTNAME'),
+        hostname=interpolate_env_var('EMAIL_HOSTNAME'),
         use_tls=True,
-        username=os.environ.get('EMAIL_USERNAME'),
-        password=os.environ.get('EMAIL_PASSWORD')
+        username=interpolate_env_var('EMAIL_USERNAME'),
+        password=interpolate_env_var('EMAIL_PASSWORD')
     )
     request.app.info_logger.info("Activation email sent", email=data['email'], username=data['username'])
 
@@ -303,7 +304,7 @@ async def login(request, payload, is_trusted=False):
     # cache actor details
     workspaces = data.pop('workspaces')
     pipe = request.app.redis_cli.pipeline()
-    pipe.hmset_dict(account_key, data)
+    pipe.hmset_dict(account_key, {key: val for key, val in data.items() if val is not None})
     if roles:
         pipe.sadd(roles_key, *roles)
     if workspace_ids:
@@ -327,8 +328,8 @@ async def login(request, payload, is_trusted=False):
     response.set_cookie(session_cookie,
                         session_token,
                         httponly=True,
-                        samesite='None',
-                        secure=True,
+                        samesite='Lax',
+                        secure=APP_MODE not in ['dev', 'test', 'demo'],
                         max_age=request.app.config.session_ttl)
     request.app.info_logger.info("User logged in", actor_id=actor_id)
     return response
@@ -1153,10 +1154,97 @@ class GetOtpSecret(AuxView):
         }
 
 
+class ActorObject(Schema):
+    username = fields.String(required=True)
+    phone = fields.String()
+    email = fields.Email(required=True)
+    password = fields.String(required=True, validate=validate.Length(min=6))
+    roles = fields.List(
+        fields.String(),
+        validate=[validators.must_not_be_empty]
+    )
+    scope = fields.String()
+    otp_secret = fields.String()
+    details = fields.Dict()
+    workspace = fields.String()
+    is_owner = fields.Boolean()
+
+
+class CreateUsers(AuxView):
+    actors = fields.List(fields.Nested(ActorObject), allow_none=False, required=True, location="body")
+
+    @summary('Create multiple users at once as admin')
+    async def post(self):
+        is_authed = 'Admin' in self.request.session.roles
+        if not is_authed:
+            raise web.HTTPForbidden()
+
+        payload = await self.validate_payload()
+        actors = payload['actors']
+        if not actors:
+            raise ValidationError('')
+
+        declared_fields = ActorObject.__dict__['_declared_fields']
+        failed = []
+        added_count = 0
+        async with self.request.app.db_pool.acquire() as conn:
+            async with conn.cursor() as cur:
+                for actor in actors:
+
+                    qobj = {field: actor.get(field) for field in declared_fields if field in actor}
+                    vals = [f'%({field})s' for field in declared_fields if field in actor]
+
+                    # clear password
+                    passwd = actor['password']
+                    salt = bcrypt.gensalt()
+                    actor['password'] = bcrypt.hashpw(passwd.encode(), salt).decode()
+
+                    # clear roles
+                    if 'roles' in actor:
+                        delta = set(actor['roles']) - self.request.app.allowed_roles
+                        if delta:
+                            bad_roles = ', '.join(delta)
+                            raise ValidationError(f'Invalid roles: {bad_roles}')
+                        actor['roles'] = Json(actor['roles'])
+
+                    # clear scope
+                    if 'scope' in actor and actor['scope'] != 'Generic':
+                        scope = actor['scope']
+                        if scope not in ScopeBase._scopes:
+                            raise ValidationError({
+                                'scope': ['Scope doesn\'t exist']
+                            })
+                    
+                    # clear details
+                    if 'details' in actor:
+                        actor['details'] = Json(actor['details'])
+                    
+                    query = f'''INSERT INTO actor (id,status,{",".join(qobj)}) VALUES (NEXTVAL('actor_id_seq'),1,{",".join(vals)}) ON CONFLICT (email) DO UPDATE SET status=1 RETURNING id'''
+
+                    await cur.execute(query, actor)
+                    if not await cur.fetchone():
+                        failed.append(actor['username'])
+                    else:
+                        added_count += 1
+        
+        if failed: 
+            msg = f"The following actors were not added: {', ',join(failed)}"
+            raise ValidationError(msg)
+                    
+        return json_response({
+            'added': added_count
+        })
+
+    class Authed:
+        class permissions:
+            post = ['Admin']
+
+
 class PrincipalActorBase(RootSchema):
     __tablename__ = 'actor'
     __aux_routes__ = {
         '/activate/email/send/': SendEmailLink,
+        '/create_users/': CreateUsers,
         '/created/activate/email/{reg_token}/': CreatedUserActivationView,
         '/activate/phone/send/': SendPhoneLink,
         '/activate/phone/{verification_code}/': PhoneActivationView,
@@ -1186,7 +1274,7 @@ class PrincipalActorBase(RootSchema):
         sqlfield=JSONB
     )
     scope = fields.String(sqlfield=sql.String(255))
-    otp_secret = fields.String(sqlfield=sql.String(24))
+    otp_secret = fields.String(sqlfield=sql.Text)
     details = fields.Dict(sqlfield=JSONB)
 
     @validates('roles')
@@ -1423,7 +1511,7 @@ class PrincipalActorBase(RootSchema):
     async def before_post(self, parent, request, data):
         if 'details' in data:
             data['details'] = data['details'].adapted
-        data['otp_secret'] = pyotp.random_base32(24)
+        data['otp_secret'] = pyotp.random_base32(160)
         query = request.query
         invitation_token = query.get('inv')
         if invitation_token is not None and invitation_token.endswith('/'):
