@@ -1,3 +1,4 @@
+import os
 import json
 import string
 import re
@@ -89,3 +90,14 @@ def seconds_to_human(ttl_seconds):
         return f'{round(ttl_minutes / 60)} hour(s)'
     else:
         return f'{ttl_minutes} minute(s)'
+
+
+def interpolate_env_var(envvar, default_value=None, template=re.compile(r'(<<(.*?)>>)')):
+    main_val = os.environ.get(envvar, default_value)
+    if not main_val:
+        return main_val
+    inter_hit = set(re.findall(template, main_val))
+    inter_dict = {templ_key: os.environ.get(key, '') for (templ_key, key) in inter_hit}
+    for key, sub in inter_dict.items():
+        main_val = main_val.replace(key, sub)
+    return main_val
\ No newline at end of file
